package com.sun.org.glassfish.gmbal;

import javax.management.MBeanInfo;
import javax.management.AttributeList;
import javax.management.InvalidAttributeValueException;
import javax.management.Attribute;
import javax.management.AttributeNotFoundException;
import javax.management.Descriptor;
import javax.management.ReflectionException;
import javax.management.IntrospectionException;
import javax.management.InstanceNotFoundException;
import javax.management.RuntimeOperationsException;
import javax.management.MBeanException;
import java.util.HashMap;
import javax.management.modelmbean.ModelMBeanInfo;
import java.util.Map;
import java.io.IOException;
import javax.management.JMException;
import javax.management.MalformedObjectNameException;
import javax.management.MBeanServerConnection;
import javax.management.ObjectName;

public class AMXClient implements AMXMBeanInterface
{
    public static final ObjectName NULL_OBJECTNAME;
    private MBeanServerConnection server;
    private ObjectName oname;
    
    private static ObjectName makeObjectName(String str) {
        try {
            return new ObjectName(str);
        }
        catch (MalformedObjectNameException ex) {
            return null;
        }
    }
    
    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (!(obj instanceof AMXClient)) {
            return false;
        }
        AMXClient other = (AMXClient)obj;
        return this.oname.equals(other.oname);
    }
    
    @Override
    public int hashCode() {
        int hash = 5;
        hash = 47 * hash + ((this.oname != null) ? this.oname.hashCode() : 0);
        return hash;
    }
    
    @Override
    public String toString() {
        return "AMXClient[" + this.oname + "]";
    }
    
    private <T> T fetchAttribute(String name, Class<T> type) {
        try {
            Object obj = this.server.getAttribute(this.oname, name);
            if (AMXClient.NULL_OBJECTNAME.equals(obj)) {
                return null;
            }
            return type.cast(obj);
        }
        catch (JMException exc) {
            throw new GmbalException("Exception in fetchAttribute", exc);
        }
        catch (IOException exc2) {
            throw new GmbalException("Exception in fetchAttribute", exc2);
        }
    }
    
    public AMXClient(MBeanServerConnection server, ObjectName oname) {
        this.server = server;
        this.oname = oname;
    }
    
    private AMXClient makeAMX(ObjectName on) {
        if (on == null) {
            return null;
        }
        return new AMXClient(this.server, on);
    }
    
    @Override
    public String getName() {
        return (String)this.fetchAttribute("Name", String.class);
    }
    
    @Override
    public Map<String, ?> getMeta() {
        try {
            ModelMBeanInfo mbi = (ModelMBeanInfo)this.server.getMBeanInfo(this.oname);
            Descriptor desc = mbi.getMBeanDescriptor();
            Map<String, Object> result = new HashMap();
            for (String str : desc.getFieldNames()) {
                result.put(str, desc.getFieldValue(str));
            }
            return result;
        }
        catch (MBeanException ex) {
            throw new GmbalException("Exception in getMeta", ex);
        }
        catch (RuntimeOperationsException ex2) {
            throw new GmbalException("Exception in getMeta", ex2);
        }
        catch (InstanceNotFoundException ex3) {
            throw new GmbalException("Exception in getMeta", ex3);
        }
        catch (IntrospectionException ex4) {
            throw new GmbalException("Exception in getMeta", ex4);
        }
        catch (ReflectionException ex5) {
            throw new GmbalException("Exception in getMeta", ex5);
        }
        catch (IOException ex6) {
            throw new GmbalException("Exception in getMeta", ex6);
        }
    }
    
    @Override
    public AMXClient getParent() {
        ObjectName res = (ObjectName)this.fetchAttribute("Parent", ObjectName.class);
        return this.makeAMX(res);
    }
    
    @Override
    public AMXClient[] getChildren() {
        ObjectName[] onames = (ObjectName[])this.fetchAttribute("Children", ObjectName[].class);
        return this.makeAMXArray(onames);
    }
    
    private AMXClient[] makeAMXArray(ObjectName[] onames) {
        AMXClient[] result = new AMXClient[onames.length];
        int ctr = 0;
        for (ObjectName on : onames) {
            result[ctr++] = this.makeAMX(on);
        }
        return result;
    }
    
    public Object getAttribute(String attribute) {
        try {
            return this.server.getAttribute(this.oname, attribute);
        }
        catch (MBeanException ex) {
            throw new GmbalException("Exception in getAttribute", ex);
        }
        catch (AttributeNotFoundException ex2) {
            throw new GmbalException("Exception in getAttribute", ex2);
        }
        catch (ReflectionException ex3) {
            throw new GmbalException("Exception in getAttribute", ex3);
        }
        catch (InstanceNotFoundException ex4) {
            throw new GmbalException("Exception in getAttribute", ex4);
        }
        catch (IOException ex5) {
            throw new GmbalException("Exception in getAttribute", ex5);
        }
    }
    
    public void setAttribute(String name, Object value) {
        Attribute attr = new Attribute(name, value);
        this.setAttribute(attr);
    }
    
    public void setAttribute(Attribute attribute) {
        try {
            this.server.setAttribute(this.oname, attribute);
        }
        catch (InstanceNotFoundException ex) {
            throw new GmbalException("Exception in setAttribute", ex);
        }
        catch (AttributeNotFoundException ex2) {
            throw new GmbalException("Exception in setAttribute", ex2);
        }
        catch (InvalidAttributeValueException ex3) {
            throw new GmbalException("Exception in setAttribute", ex3);
        }
        catch (MBeanException ex4) {
            throw new GmbalException("Exception in setAttribute", ex4);
        }
        catch (ReflectionException ex5) {
            throw new GmbalException("Exception in setAttribute", ex5);
        }
        catch (IOException ex6) {
            throw new GmbalException("Exception in setAttribute", ex6);
        }
    }
    
    public AttributeList getAttributes(String[] attributes) {
        try {
            return this.server.getAttributes(this.oname, attributes);
        }
        catch (InstanceNotFoundException ex) {
            throw new GmbalException("Exception in getAttributes", ex);
        }
        catch (ReflectionException ex2) {
            throw new GmbalException("Exception in getAttributes", ex2);
        }
        catch (IOException ex3) {
            throw new GmbalException("Exception in getAttributes", ex3);
        }
    }
    
    public AttributeList setAttributes(AttributeList attributes) {
        try {
            return this.server.setAttributes(this.oname, attributes);
        }
        catch (InstanceNotFoundException ex) {
            throw new GmbalException("Exception in setAttributes", ex);
        }
        catch (ReflectionException ex2) {
            throw new GmbalException("Exception in setAttributes", ex2);
        }
        catch (IOException ex3) {
            throw new GmbalException("Exception in setAttributes", ex3);
        }
    }
    
    public Object invoke(String actionName, Object[] params, String[] signature) throws MBeanException, ReflectionException {
        try {
            return this.server.invoke(this.oname, actionName, params, signature);
        }
        catch (InstanceNotFoundException ex) {
            throw new GmbalException("Exception in invoke", ex);
        }
        catch (IOException ex2) {
            throw new GmbalException("Exception in invoke", ex2);
        }
    }
    
    public MBeanInfo getMBeanInfo() {
        try {
            return this.server.getMBeanInfo(this.oname);
        }
        catch (InstanceNotFoundException ex) {
            throw new GmbalException("Exception in invoke", ex);
        }
        catch (IntrospectionException ex2) {
            throw new GmbalException("Exception in invoke", ex2);
        }
        catch (ReflectionException ex3) {
            throw new GmbalException("Exception in invoke", ex3);
        }
        catch (IOException ex4) {
            throw new GmbalException("Exception in invoke", ex4);
        }
    }
    
    public ObjectName objectName() {
        return this.oname;
    }
    
    static {
        NULL_OBJECTNAME = makeObjectName("null:type=Null,name=Null");
    }
}
